var xmldoc = require('xmldoc')
var template = require('art-template')
require('./helper')(template)
var fs = require('fs');
var cache = require('memory-cache');
var ajax = require('mjax');
var mlog = require('mqlog');
var marr = require('marr');
var natis = require('./natis');
var mvalidate = require('mvalidate');
require('mstr');

var uuidLen = 8;
var pool = {};

//给不同的connection设置不同的参数
!function(){
    var util = {
        config : function(config){
            var connName = config.jdbc.connName;
            pool[connName] = {};
            pool[connName]['jdbc']  = config.jdbc || {};
            pool[connName]['mapper'] = config.mapper;
            pool[connName]['validate'] = config.validate || {};
            uuidLen = config.jdbc.uuid_len || 8;
        },
        errs : {
            uniqueError : '唯一性错误',
            checkError : '删除时检查错误',
            insertError : '插入失败',
            updateError : '更新失败',
            deleteError : '删除失败'
        },
        getExecsql : function(dbModel,model,method,param,resolve,reject,res){
            //为每次查询生成一个id,便于排查问题
            dbModel._id = Date.now()
            var jdbc = pool[dbModel.connName]['jdbc'];
            var validate = pool[dbModel.connName]['validate'];
            var mapperDir = pool[dbModel.connName]['mapper'] ;
            var key = model + "." + method;
            //先读缓存
            var cacheSql = cache.get(key);
            if(jdbc.debug){
                mlog.info(dbModel._id, "读取模板:", key)
            }
            var presql = cacheSql;
            if(!cacheSql){
                var path = mapperDir + '/'+ model+'.xml';
                if(!fs.existsSync(path)){
                    mlog.error(path + "文件不存在");
                    return util.reject('系统错误',reject,res);
                };
                var presqlfile = fs.readFileSync(path,{encoding:"utf-8"});
                var document = new xmldoc.XmlDocument(presqlfile);
                var methodEle = document.childWithAttribute("id",method)

                if(!methodEle || !methodEle.val){
                    mlog.error(model+"的"+method+"方法未定义");
                    return util.reject('系统错误',reject,res);
                }
                var presql = document.childWithAttribute("id",method).val;
                //写入缓存
                cache.put(key,presql,1000 * 60 * 5);
            }
            if(jdbc.debug){
                mlog.info(dbModel._id, "模板SQL:", presql);
            }
            if(jdbc.debug){
                mlog.info(dbModel._id, "传入参数:", param);
            }
            var sql = '';
            try{
                 sql = template.compile(presql,{compress:true,escape:false})(param);
            }
            catch(err){
                mlog.error(dbModel._id, "SQL模板编译出错");
                return util.reject("系统错误",reject,res);
            }

            sql = sql.trim();
            //处理若干异常SQL
            //where and
            sql = sql.replace(/where\s+and/g, 'where ');
            //where )
            sql = sql.replace(/where\s+\)/g, ')')
            //where结尾
            if(sql.toLowerCase().lastIndexOf("where") == sql.length - 5){
                console.log("空where")
                sql = sql.slice(0, -5);
            }
            if(jdbc.debug){
                mlog.info(dbModel._id, "执行SQL:", sql);
            }
            return sql;
        },
        reject : function(e,reject,res){
            if(res){
                if(typeof res == "function"){
                    return res(e);
                }
                return ajax.failure(res,e);
            }
            return reject(e);
        },
        resolve : function(data,resolve,res){
            if(res){
                if(typeof res == "function"){
                    return res(null,data);
                }
                return ajax.success(res,data);
            }
            return resolve(data);
        },
        exec : function(dbModel,model,method,param,resolve,reject,res,ele,deal){

            /*前置操作
             1.如果是删除操作,则进行合法性判断
             2.如果是分页查询操作,则要把查询总量也要返回

             后置数据处理
             1.如果对于非列表查询,则将返回的数据进行处理
             2.如果一对多查询,那么在查询后要处理成一对多的形式

             */
            var beforeMethod = ele.attr.before;
            var args = arguments;
            if(beforeMethod){ //需要前置处理
                //先判断是否存在此方法
                var bean = (dbModel.models)[model];
                if(bean.hasOwnProperty(beforeMethod)){
                    bean[beforeMethod](param).then(function(){
                        //继续执行
                        util.justDeal.apply(util,args);
                    },function(err){
                        mlog.error("前置处理出现错误:");
                        mlog.error(err);
                        return util.reject(err,reject,res);
                    });
                }
                else{
                    mlog.error("前置方法未定义,忽略执行");
                    util.justDeal.apply(util,args);
                }
            }
            else{
                util.justDeal.apply(util,args);
            }
        },
        justDeal : function(dbModel,model,method,param,resolve,reject,res,ele,deal,times){
            times = times || 0;
            var jdbc = pool[dbModel.connName]['jdbc'];
            var validate = pool[dbModel.connName]['validate'];
            var mapperDir = pool[dbModel.connName]['mapper'];
            var sql = this.getExecsql(dbModel,model,method,param,resolve,reject,res);
            //取新连接
            var connName = dbModel.connName
            natis.getConnection(connName, function(db){
                db.querying = true
                db.query({sql : sql,timeout : 10000},function(err,data){
                    db.querying = false;
                    //查询失败
                    if(err){
                        //判断失败类型
                        if(err.message.startsWith("ER_PARSE_ERROR")){
                            mlog.error(dbModel._id,"SQL错误" + sql);
                            return util.reject("服务器错误",reject,res);
                        }
                        else if(err.message.startsWith("ER_TRUNCATED_WRONG_VALUE_FOR_FIELD")){
                            mlog.error(dbModel._id,"SQL字段赋值错误" + err.message);
                            return util.reject("服务器错误",reject,res);
                        }

                        times ++;
                        mlog.error(dbModel._id, err);
                        //重新获取连接，并重新操作
                        // mlog.info("准备重试...");
                        // // dbModel.connection.disabled = true;
                        // mlog.info(dbModel.connName);
                        //
                        // natis.getDb(dbModel.connName, function(db){
                        //     if(times == 3){
                        //         mlog.error(err);
                        //         if(jdbc.debug){
                        //             mlog.error("数据操作失败");
                        //             mlog.error(err);
                        //         }
                        //         return util.reject("服务器错误",reject,res);
                        //     }
                        //     util.justDeal.call(util, db,model,method,param,resolve,reject,res,ele,deal,times);
                        // });
                        return util.reject("服务器错误", reject, res)
                    }

                    //调用回调方法处理MYSQL返回结果
                    try{
                        data = deal(data,method,ele);
                    }catch (e){
                        var msg = e.message;
                        switch (msg){
                            case util.errs.uniqueError :
                                if(validate[model] && validate[model].unique) {
                                    msg = validate[model].unique;
                                }
                                else if(ele.attr.msg){
                                    msg = ele.attr.msg;
                                }
                                else{
                                    msg = "存在相同记录";
                                }
                                break;
                            case util.errs.checkError:
                                if(validate[model] && validate[model].check) {
                                    msg = validate[model].check;
                                }
                                else if(ele.attr.msg){
                                    msg = ele.attr.msg;
                                }
                                else{
                                    msg = "该记录已被关联,无法删除";
                                }
                                break;
                            default :
                                break;
                        }
                        return util.reject(msg,reject,res);
                    }
                    if(jdbc.debugData){
                        mlog.info(dbModel._id,"处理结果:", data)
                    }
                    return util.resolve(data,resolve,res);
                });
            })
        },
        select : function(){
            util.exec.apply(util,util.getArgs(arguments,function(data,method,ele){
                if(data.length == 0){
                    return null;
                }
                else if(ele.attr.tree){
                    data = marr.generateTreeByPid(data,ele.attr.idKey,ele.attr.pidKey,ele.attr.pidValue);
                }
                else if(ele.attr.child){
                    data = marr.generateTreeByCatalog(data,ele.attr.child,ele.attr.key,ele.attr.childName);
                }
                if(data == null || data.length == 0){
                    return null;
                }
                if(ele.attr.one || method == 'get'){
                    data = data[0]
                }
                if(data.length == 1 && util.isCountSql(data[0])){
                    data = data[0]
                }
                //结果集只有一个元素
                if(!Array.isArray(data) && util.countEle(data) == 1 && (ele.attr.one || util.isCountSql(data))){
                    data = util.getFirstEle(data);
                }
                if(method.startsWith("isUnique","unique")){
                    if(data > 0){
                        throw new Error(util.errs.uniqueError);
                    }
                }
                else if(method.startsWith("isCheck","check")){
                    if(data > 0){
                        throw new Error(util.errs.checkError);
                    }
                }
                return data;
            }));
        },
        insert : function(dbModel,model,method,param,resolve,reject,res,ele){

            // user : {  //key
            //     'name' : { // col
            //         minlen : {val:5,message:'用户名至少6位'},  //ruleItem = ruleValue
            //         require : {val:true,messagge:'用户名不能为空'} //ruleItem = ruleValue
            //     },
            // }
            //验证
            var jdbc = pool[dbModel.connName]['jdbc'];
            var validate = pool[dbModel.connName]['validate'];
            var mapperDir = pool[dbModel.connName]['mapper'] ;
            var validateModel = validate[model];

            if(validateModel){
                //验证
                if(jdbc.debug){
                    mvalidate.setDebug(true);
                }
                var validateOk = true;
                var msg = '';
                for(var col in validateModel){
                    var colVal = param[col]; //取值
                    var rules = validateModel[col]; //取规则
                    if(typeof rules == "string"){
                        continue;
                    }
                    var validateErrorItem = '';
                    for(var ruleItem in rules){
                        var ruleVal = rules[ruleItem]['val']; //取规则值
                        validateErrorItem = ruleItem;
                        if(ruleItem == "required" && ruleVal && (colVal == null || colVal == '')){
                            validateOk = false;
                            break;
                        }
                        if(ruleItem != "required" && !mvalidate.validate(ruleItem,colVal,ruleVal)){
                            validateOk = false;
                            break;
                        }
                    }
                    if(!validateOk){
                        if(rules[validateErrorItem]['message']){
                            msg = rules[validateErrorItem]['message'];
                            mlog.error("字段验证失败:"+ msg);
                        }
                        else{
                            mlog.error('缺少字段' + validateErrorItem + '验证失败提示信息配置');
                            msg = '字段信息校验失败';
                        }
                        return util.reject(msg,reject,res);
                    }
                }
            }

            if(jdbc.uuid_len && !param.uuid){
                param.uuid = String.random(uuidLen);
            }
            
            util.exec.apply(util,util.getArgs(arguments,function(data,method,ele){
                return data.affectedRows;
            }));
        },
        update : function(dbModel,model,method,param,resolve,reject,res,ele){
            var jdbc = pool[dbModel.connName]['jdbc'];
            var validate = pool[dbModel.connName]['validate'];
            var mapperDir = pool[dbModel.connName]['mapper'] ;
            var validateModel = validate[model];

            if(validateModel){
                //验证
                if(jdbc.debug){
                    mvalidate.setDebug(true);
                }
                var validateOk = true;
                var msg = '';
                for(var col in validateModel){
                    var colVal = param[col]; //取值
                    var rules = validateModel[col]; //取规则
                    if(typeof rules == "string"){
                        continue;
                    }
                    var validateErrorItem = '';
                    for(var ruleItem in rules){
                        var ruleVal = rules[ruleItem]['val']; //取规则值
                        validateErrorItem = ruleItem;

                        if(ruleItem == "required" && ruleVal && (colVal == '')){
                            validateOk = false;
                            break;
                        }
                        if(ruleItem != "required" && !mvalidate.validate(ruleItem,colVal,ruleVal)){
                            validateOk = false;
                            break;
                        }
                    }
                    if(!validateOk){
                        if(rules[validateErrorItem]['message']){
                            msg = rules[validateErrorItem]['message'];
                            mlog.error("字段验证失败:"+ msg);
                        }
                        else{
                            mlog.error('缺少字段' + validateErrorItem + '验证失败提示信息配置');
                            msg = '字段信息校验失败';
                        }
                        return util.reject(msg,reject,res);
                    }
                }
            }


            util.exec.apply(util,util.getArgs(arguments,function(data,method,ele){
                return data.affectedRows;
            }));
        },
        delete : function(){
            util.exec.apply(util,util.getArgs(arguments,function(data,method,ele){
                return data.affectedRows;
            }));
        },
        countEle : function(o){
            if(typeof o == "object"){
                var sum = 0;
                for(var i in o){
                    sum ++;
                }
                return sum;
            }
            return -1;
        },
        getFirstEle : function(o){
            if(typeof o == "object"){
                for(var i in o){
                    return o[i];
                }
            }
            return -1;
        },
        isCountSql : function(o){
            for(var key in o){
                return key.toLowerCase().indexOf("count(") == 0;
            }
            return false;
        },
        getArgs : function(args,other1,other2){
            var arr = [];
            for(var i in args){
                arr[i] = args[i];
            }
            if(typeof other1 != "undefined"){
                arr.push(other1);
            }
            if(typeof other2 != "undefined"){
                arr.push(other2);
            }
            return arr;
        }
    }
    module.exports = util;

}();
