var xmldoc = require('xmldoc');
var fs = require('fs');
var Promise = require('bluebird');
var mlog = require('mqlog');
var path = require('path');
var dbutil = require('./dbutil');
module.exports = function(dbModel,config){

    var mapperDir = config.mapper;
    dbutil.config(config);

    mlog.info("start init model of " + dbModel.connName  + " : " + new Date().toLocaleString());
    //遍历mapper目录下的所有文件
    mlog.info("读取路径：" +fs.realpathSync(mapperDir))
    var files = fs.readdirSync(mapperDir);

    for(var i in files){
        //只解析xml文件
        if(files[i].indexOf(".xml") < 0){
            continue
        }
        mlog.info(files[i]);
        var presqlfile = fs.readFileSync(mapperDir + '/'+files[i],{encoding:"utf-8"});
        var document = new xmldoc.XmlDocument(presqlfile);
        //获取根节点的名称,即对象名称
        var table = "";
        try{
            table = document.attr.id;
            if(table == ""){
                mlog.error("请设置xml所属对象");
                return;
            }
        }
        catch(e){
            mlog.error("请设置xml所属对象");
            return;
        }

        var obj = {};
        document.eachChild(function(ele){
            var operation = ele.name;
            var method = "";
            try{
                method = ele.attr.id;
            }
            catch(e){
                mlog.error("请设置xml节点方法名");
                return;
            }
            if(method == ''){
                mlog.error("请设置xml节点方法名");
                return;
            }

            obj[method] = (function(table,ele){
                var method = ele.attr.id;
                return function(param,res){
                    //简单的讲：只有传入param

                    //可能传入以下几种格式：
                    //0、不传任何参数
                    //1、param
                    //2、res/cb
                    //3、param,res/cb

                    //不传任何参数
                    if(arguments.length == 0){
                        param = {};
                    }
                    //只传递param
                    else if(arguments.length == 1 && typeof param != "function"){
                        res = null;
                    }
                    //只传递res/cb
                    else if(arguments.length == 1 && (param.hasOwnProperty("domain") || typeof param == "function")){
                        res = param;
                        param = {};
                    }
                    //param,res/cb都传
                    return new Promise(function(resolve,reject){
                        switch(operation){
                            case "select":
                            case "insert":
                            case "update":
                            case "delete":
                                dbutil[operation](dbModel,table,method,param,resolve,reject,res,ele);
                                break;
                            default :
                                mlog.error("非法标签");
                                break;
                        }
                    });
                }
            })(table,ele);
        });

        (dbModel.models)[table] = obj;
    }
    mlog.info("init model end of " + dbModel.connName  + " : " +new Date().toLocaleString());
}
