var mysql = require('mysql');
var Promise = require('bluebird');
var mlog = require('mqlog');
var marr = require('marr')
var config = {
	'mapper' : '/db/mapper',
	'jdbc' : {},
	'validate' : {}
}
//连接池
var connPool = {};
//对象定义
var dbModel = null;
//其它数据源定义
var otherModels = null;
//定时器
var timer = {};
//数据库备份配置
var maxPoolNum = 10;
var backUpJdbcConfig = {};
var natis = {
	config(name, value){
		config[name] = value;
	},
	init(cb){
		var connName = config.jdbc.connName || 'conn_auto';
		if(dbModel && dbModel.connName){
			if(cb){
				return cb(null, dbModel)
			}
			return;
		}
		//随机产生若干连接
		config.jdbc.connName = connName;
		makeRandomConn(config.jdbc);
		maxPoolNum = config.jdbc.maxPoolNum || 10
		var tmpTimer = setInterval(function(){
			if(!connPool[connName] || connPool[connName].length === 0){
				console.log("连接池还空空如也,继续等待...");
				return;
			}
			clearInterval(tmpTimer);
			//初始化数据对象
			dbModel = {
				connName : connName, // 设置数据库连接名(每个数据库都保证唯一)
				models : {}
			};
			//根据数据库配置解析SQL
			require('./init')(dbModel,config);
			if(cb){
				return cb(null, dbModel)
			}
		}, 200);
	},
	use(cb){
		this.init(cb);
	},
	//支持任意源数据源
	connect(config,cb){
		var connName = config.jdbc.connName;
		var mModel = getOtherModels(connName);
		if(mModel && mModel.connName){
			if(cb){
				return cb(null, mModel)
			}
			return;
		}
		//随机产生若干连接
		makeRandomConn(config.jdbc);

		mModel = {
			connName : connName, // 设置数据库连接名(每个数据库都保证唯一)
			models : {}
		};
		//根据数据库配置解析SQL
		require('./init')(mModel,config);
		//push进去
		otherModels = otherModels || [];
		otherModels.push(mModel);
		if(cb){
			//随机获取一个连接
			return cb(null, mModel)
		}
	},
	check(config, cb){
		var db = mysql.createConnection(config);
		db.connect(function(err){
			if(err){
				return cb(err)
			}
			return cb()
		});
	},
	db(cb){
		var connName = config.jdbc.connName || 'conn_random';
	    this.getDb(connName,cb);
	},
	getDb(connName,cb){
		if(dbModel && dbModel.connName){
			//随机获取一个连接
			return cb(dbModel);
		}
		throw new Error("数据库实例获取失败，请检查连接名是否正确")
		cb(null);
	},
	getConnection(connName, cb){
		getRandomConn(connName, cb)
	}
};


function makeRandomConn(jdbc, cb){
	var newConn = 0;
	var mysqlConfig = null;
	var connName = '';
	var poolNum = 0;
	if(typeof jdbc == "string"){
		connName = jdbc;
		mysqlConfig = backUpJdbcConfig[connName];
		if(!jdbc){
			mlog.error("执行该操作前，当前至少已经创建了一个连接");
			throw new Error();
			return;
		}
		newConn = 1;
		poolNum = 1;
	}
	else{
		connName = jdbc.connName || 'conn_random';
		mysqlConfig = jdbc.conn || jdbc.connection();
		//配置备份
		backUpJdbcConfig[connName] = mysqlConfig;
		//随机创建若干连接
	    poolNum = jdbc.poolNum || 3;
	}
	var poolConns = connPool[connName];
	if(poolConns && poolConns.length && !newConn){
		return;
	}
	//只允许一次回调
	var canCb = true;
	for(var i=0;i<poolNum;i++){
		var db = mysql.createConnection(mysqlConfig);
		db.connect(function(err){
			if(err){
				console.error(mysqlConfig)
				throw new Error("数据库连接失败, 请检查MYSQL是否启动，或者配置是否正确")
				return;
			}
			console.log("创建连接成功【连接池】")
			var dbp = {
					db : db,
					name : "conn_" + Date.now()
				};
			var conns = connPool[connName];
			if(conns && conns.length){
				conns.push(dbp);
			}
			else{
				connPool[connName] = [dbp];
			}
			if(newOne && cb && canCb){
				canCb = false;
				cb(db);
			}
		});
	}

	//如果存在定时器，则忽略
	if(timer[connName]){
		return;
	}

	const keepTime = jdbc.keepTime || 1
	//添加定时器
	timer[connName] = setInterval(function(){
		var pools = connPool[connName];
		if(pools && pools.length){
			for(var i in pools){
				//间歇调用
				(function(j){
					setTimeout(function(){
						keepConnection(pools[j]);
					}, 1000 * i);
				})(i);
			}
		}
	},1000 * 60 * keepTime);
}

//根据数据库连接名随机获取一个连接
function getRandomConn(connName, cb){
	var conns = connPool[connName];
	if(conns && conns.length){
		//随便找一个没有在查询的连接
		var dbIndex = -1;
		for(var i in conns){
			if(!conns[i].db.querying){
				dbIndex = i;
				break;
			}
		}
		console.log("取连接 " + dbIndex)
		if(dbIndex == -1){ //没有空余的连接
			if(conns.length < maxPoolNum){
				console.log("创建新的连接。。。。")
				//创建连接
				return makeRandomConn(connName, cb); //需要新增连接
			}
			else{
				//等一会吧
				setTimeout(function(){
					getRandomConn(connName, cb)
				}, 1000)
			}
		}
		else{
			return cb(conns[dbIndex].db)
		}
	}
	//等待片刻即可
	setTimeout(function(){
		getRandomConn(connName, cb)
	}, 50);
}

function newOne(connName){
	if(!backUpJdbcConfig[connName]){
		return null;
	}
	var db = mysql.createConnection(mysqlConfig);
	db.connect(function(err){
		if(err){
			mlog.error("创建连接失败【连接池新连接】 " + err.messsage);
			return;
		}
		mlog.info("创建连接成功【连接池新连接】")
		return db;
	});
}



//- Reconnection function
function keepConnection(poolOne){
	if(poolOne && poolOne.db.querying){
		console.log("旧连接正在查询1...");
		return;
	}
	poolOne.db.querying = true;
	poolOne.db.query("select 1", function(err, data){
		if(err){
			console.error("连接保持失败，可能出现数据库连接错误【定时刷新】", err, new Date().toLocaleString());
			setTimeout(function(){
				keepConnection(poolOne);
			}, 3000);
		}
		else{
			poolOne.db.querying = false;
			console.log("连接保持成功【定时刷新】", new Date().toLocaleString());
		}
	})
}

function getOtherModels(connName){
	if(!otherModels){
		return null;
	}
	return otherModels.filter(function(val, index){
		if(val.connName == connName){
			return val;
		}
	});
}

module.exports = natis;
